#include "parser.h"
#include <string.h>

cardType::cardType()
{
	numFields = 0;
	name = NULL;
	notes = NULL;
	unknowns=false;
	untext=NULL;
}

void cardType::add(char *s, char *t)
{
	//printf("attemping to add \"%s\" and \"%s\"\n", s ,t );
	char * a = new char[strlen(s)+1];
	char * b = new char[strlen(t)+1];
	strcpy(a,s);
	strcpy(b,t);
	//printf("addin \"%s\" and \"%s\"\n", a ,b );
	names.push_back(a);
	data.push_back(b);
	numFields++;
}

void cardType::setName(char *s)
{
	if(name != NULL)
		delete name;
	name = new char[strlen(s)+1];
	strcpy(name,s);
}

char * cardType::getName(int i)
{
	if(i>numFields || i<0)
		return "ERROR!";
	else
		return names.at(i);
}

char * cardType::getData(int i)
{
	if(i>numFields || i<0)
		return "ERROR!";
	else
		return data.at(i);
}

void cardType::setNotes(char *s)
{
	if(notes != NULL)
		delete notes;
	notes = new char[strlen(s)+1];
	strcpy(notes,s);
}

void cardType::setUnknowns(char *s)
{
	if(untext != NULL)
		delete untext;
	untext = new char[strlen(s)+1];
	strcpy(untext,s);
	unknowns=true;
}

/******************************************************* PARSER!
The following functions are used in the parser, to extract the
different fields from the magstripe stream. These will go in a
different file later on*/

bool isDelim(char ch, char* delim)
{
  if (ch == 0) return 1;/*end of string is always delim*/
  while(*delim)
    {
      if (*delim++ == ch) return true;
    }
  return false;
}

int extractFields(char* st, char** subSt, int lth, char* delim)
{
	char* st1 = st;
	int i = 0;
	char* start;
    char found;

	if (*st == 0) return 0; /*Empty string, no substrings*/
	while(i < lth) {
		start = st;
		while(1) {
			if (isDelim(*st, delim)) {
				/* Found end of string*/
				found = *st; /*record the terminating character*/
				*st++ = 0;   /* Terminate the substring*/

				subSt[i] = start; /*Start of this substring*/
				/*are we at the end of the string?*/

				if(found == 0) return i;
				/*continue going*/
				i++;
				break;
			} else {
				st++; /*next*/
			}
		}
	}
	return i; /*prevents overflow of subSt array*/
}

track2 extractTrack2(char * card) {

	track2 t2;

	if ( (t2.contents = (char *) malloc(strlen(card) + 1)) == NULL) {
		printf("Error: extractTrack2(): Cannot malloc space!\n");
		exit(1);
	}
	if ( (t2.divided = (char *) malloc(strlen(card) + 1)) == NULL) {
		printf("Error: extractTrack2(): Cannot malloc space!\n");
		exit(1);
	}

	strcpy(t2.contents, card);
	strcpy(t2.divided, t2.contents);
	t2.divided++;
	/*parse it!*/
	t2.numFields = extractFields(t2.divided, t2.fields, 40, TRACK2DELIM);
	return t2;

}

// ================================================================= TESTS

bool mod10check(char *card)
{
	unsigned char tmp, sum, i, j;
	sum = i = j = 0;
	/*count the card digits*/
	while(card[i] >= '0' && card[i] <= '9')
		i++;
	while(i--)
	{
	        tmp = card[i] & 0x0F;
		if(j++ & 1)
		{
			if((tmp<<=1) > 9)
			{
				tmp -= 10;
				sum++;
			}
		}
		sum += tmp;
	}
	if(sum % 10 == 0)
		return true;
	return false;
}

bool isMonth(char * d)
{
	char tmp[3] = {0,0,0};
	int i;
	for(i=0;i<2;i++)
		if(d[i]<48 || d[i]>57)
			return false;
	strncpy(tmp,d,2);
	i=atoi(tmp);
	if(i>=1 && i<=12) return true;
	return false;
}

int lastDotm(int m, int y)
{
	int dotm[12] = { 31, 31, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
	bool leap=false;

	m--;
	if(m!=1)
		return dotm[m];

	if(y % 4 ==0) {
		leap=true;
		if(y % 100 == 0) {
			leap = false;
			if(y % 400 ==0)
				leap = true;
		}
	}
	return (leap) ? 29: 28;
}

bool dateValid(char *m, char *d, char * y)
{

	char day[3] = {0,0,0};
	char month[3]={0,0,0};
	char year[5]={0,0,0,0,0};
	int i;

	bool leap=false;

	if(!isMonth(m)) return false;
	for(i=0;i<2;i++)
		if(d[i]<48 || d[i]>57)
			return false;
	for(i=0;i<4;i++)
		if(y[i]<48 || y[i]>57)
			return false;
	strncpy(day,d,2);
	strncpy(month,m,2);
	strncpy(year,y,4);
	i=atoi(day);
	if(i<=0 || i>=lastDotm(atoi(month), atoi(year)))
		return false;

	return true;
}

int expandYear(int i)
{
	//try and expand a 2 digit YY to a CCYY
	if(i>=35 && i<=99)
		i+=1900;
	else
		i+=2000;
	return i;
}

// =============================================================== FORMATTERS

char * monthName(int x)
{
	switch(x)
	{
		case 1:
			return "January";
			break;
		case 2:
			return "February";
			break;
		case 3:
			return "March";
			break;
		case 4:
			return "April";
			break;
		case 5:
			return "May";
			break;
		case 6:
			return "June";
			break;
		case 7:
			return "July";
			break;
		case 8:
			return "August";
			break;
		case 9:
			return "September";
			break;
		case 10:
			return "October";
			break;
		case 11:
			return "November";
			break;
		case 12:
			return "December";
			break;
		default:
			return "ERROR!";
	}
}

char * formatter(char * format, char * n)
{
	int i=0;
	int k=0;
	char * foo = new char[strlen(format)+1];
	for(i=0;i<strlen(format);i++)
	{
		if(format[i]=='X') {
			foo[i]=n[k];
			k++;
		}
		else
			foo[i]=format[i];
	}
	return foo;
}

// ================================================================= 1 Field

//Checks for Barnes and Noble Reader Advantage Card
//500XXXXXXX
cardType bnreader(track2 t2)
{
	cardType card;

	//has 10 characters
	if(strlen(t2.fields[0])!=10) return card;;
	//has 500 constant
	if(strncmp(t2.fields[0],"500",3)!=0) return card;;
	//WE ARE GOOD!
	card.setName("Barnes and Noble Reader's Advantage Card");
	card.add("Membership Number",formatter("XXX-XXXX-XXX",t2.fields[0]));

	return card;
}

//Radisson Gold Rewards Card
//6015XXXXXXXXXXXX 16 digits
cardType radisson(track2 t2)
{

	cardType card;

	//1st is 16
	if(strlen(t2.fields[0])!=16) return card;
	//starts with a 6015
	if(strncmp(t2.fields[0],"6015",4)!=0) return card;

	//WE ARE GOOD!
	card.setName("Radisson Gold Rewards Card");
	card.add("Membership Number",formatter("XXXX XXXX XXXX XXXX",t2.fields[0]));

	return card;
}

// ================================================================= 2 Fields

//Checks for Visa Credit Card
//4xxxxxxxxxxxxxx@=YYMM101xxxxxxxx 16 digits
//4xxxxxxxxxxx@=YYMM101xxxxxxxx 13 digits
cardType visa(track2 t2)
{
	char * tmp;
	FILE * fin;
	char tmp2[80];
	cardType card;
	bool flag;
	memset(tmp2,0,80);

	//is 16 or 13 characters
		if(strlen(t2.fields[0]) !=16 && strlen(t2.fields[0])!=13) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 101 constant
	tmp=t2.fields[1];
	if(strncmp(&tmp[4],"101",3)!=0) return card;
	//starts with a 4
	if(t2.fields[0][0]!='4') return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][2])) return card;

	//WE ARE GOOD!
	card.setName("Visa Credit Card");
	card.add("Account Number",formatter("XXXX XXXX XXXX XXXX",t2.fields[0]));
	//Get the expiration month
	strncpy(tmp2,&tmp[2],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, tmp[0], tmp[1]);
	card.add("Expires",tmp2);
	memset(tmp2,0,80);
	//get E-pin
	strncpy(tmp2,&tmp[8],7);
	card.add("Encrypted PIN",tmp2);
	/*get issuing bank!*/
	if( (fin=fopen(VISADATABASE,"r")) == NULL) {
		printf("Error: visa(): Cannot open \"%s\"\n",VISADATABASE);
	} else {
		tmp = new char[80];
		memset(tmp,0,80);
		memset(tmp2,0,80);
		flag = false;
		while(!feof(fin))
		{
			fgets(tmp2,80,fin);
			if(strlen(tmp2)>6) {
				if( strncmp(t2.fields[0],tmp2,4) == 0) {
					strcpy(tmp, &tmp2[5]);
					flag = true;
					break;
				}
			}
		}
		if(flag)
			card.add("Issuing Bank", &tmp2[5]);
		else {
			card.add("Issuing Bank", "Unknown");
			card.setUnknowns("Issuing Bank Prefix is unknown");
		}
		fclose(fin);
		delete [] tmp;
	}

	return card;
}

//American Express Credit Card
//34xxxxxxxxxxxx@=YYMM101xxxxxxxx BLUE 15 digits, starts 34 or 37
//34xxxxxxxxxxxx@=YYMMxxxxxxxx regular 15 digits, starts 34 or 37
cardType amex(track2 t2)
{
	char tmp2[80];
	cardType card;
	memset(tmp2,0,80);
	bool flag=true;

	//1st is 15 characters
	if(strlen(t2.fields[0]) !=15) return card;
	//2nd has >=12 characters
	if(strlen(t2.fields[1]) <12) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 34 or 37 prefix!
	if(strncmp(t2.fields[0],"34",2)!=0 &&
	 strncmp(t2.fields[0],"37",2)!=0) return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][2])) return card;

	//WE ARE GOOD!

	if(strncmp(&t2.fields[1][4],"101",3)==0) {
		card.setName("American Express Blue Card");
		flag=false;
	} else
		card.setName("American Express Credit Card");

	card.add("Account Number",formatter("XXXX XXXXXX XXXXX",t2.fields[0]));
	//Get the expiration month
	strncpy(tmp2,&t2.fields[1][2],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, t2.fields[1][0], t2.fields[1][1]);
	card.add("Expires",tmp2);

	if(flag) card.setUnknowns("Card Type (Gold card, Green card)");

	return card;
}

//Checks for AAA card
//4xxxxxxxxxxxxxx@=YYMM101xxxxxxxxx
cardType AAA(track2 t2)
{
	char tmp2[80];
	cardType card;
	memset(tmp2,0,80);

	//1st is 16 characters
	if(strlen(t2.fields[0])!=16) return card;;
	//snd is 20 characters
	if(strlen(t2.fields[1])!=20) return card;;
	//first character is a 4
	if(strncmp(t2.fields[0],"4",1)!=0) return card;;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 101 constant
	if(strncmp(&t2.fields[1][4],"101",3)!=0) return card;
	//check for "4976" at end of field 2
	if(strncmp(&t2.fields[1][16],"4976",4)!=0) return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][2])) return card;

	//WE ARE GOOD!
	card.setName("American Automobile Association Membership Card");
	card.add("Account Number",formatter("XXX XXX XXXXXXXXX",t2.fields[0]));
	//Get the expiration month*/
	strncpy(tmp2,&t2.fields[1][2],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, t2.fields[1][0], t2.fields[1][1]);
	card.add("Expires",tmp2);

	card.setUnknowns("State it was issued in, Level of Membership?, Member Since");

	return card;
}

//Checks for Barnes and Noble Gift card
//5045xxxxxxxxxxx=xxxx...
cardType bngift(track2 t2)
{
	cardType card;

	//1st is 15 characters
	if(strlen(t2.fields[0])!=15) return card;;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 5045 constant
	if(strncmp(t2.fields[0],"5045",4)!=0) return card;;

	//WE ARE GOOD!
	card.setName("Barnes and Noble Giftcard");
	card.add("Account Number",t2.fields[0]);

	card.setUnknowns("Store Number or State it was issued in, Original Value");
	return card;
}

//Checks for Kroger card
//603xxxxxxxxxxxx=xxx sometimes =xxxx
cardType kroger(track2 t2)
{
	cardType card;

	//1st is 15 characters
	if(strlen(t2.fields[0])!=15) return card;;
	//snd is at least 3 characters
	if(strlen(t2.fields[1])<3) return card;;
	//has 603 constant
	if(strncmp(t2.fields[0],"603",3)!=0) return card;;

	//WE ARE GOOD!
	card.setName("Kroger Plus Shopping Card");
	//UNKNOWN
	card.add("Account Number",t2.fields[0]);
	card.setUnknowns("Store Number or State it was issued in");
	return card;
}

//Generic ATM Card
//xxxxxxxxxxxxxx@=YYMM101xxxxxxxx 16 digits
cardType atm(track2 t2)
{
	char tmp2[80];
	cardType card;
	memset(tmp2,0,80);

	//is 16 characters
	if(strlen(t2.fields[0]) !=16) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 120 constant
	if(strncmp(&t2.fields[1][4],"120",3)!=0) return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][2])) return card;

	//WE ARE GOOD!
	card.setName("Generic ATM/Banking Card");
	card.add("Account Number",formatter("XXXX XXXX XXXX XXXX",t2.fields[0]));
	//Get the expiration month
	strncpy(tmp2,&t2.fields[1][2],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, t2.fields[1][0], t2.fields[1][1]);
	card.add("Expires",tmp2);
	memset(tmp2,0,80);
	//get E-pin
	strncpy(tmp2,&t2.fields[1][8],7);
	card.add("Encrypted PIN",tmp2);

	return card;
}

//BP Gas Card
//7211xxxxxxxxxx=YYMMxxxxxxxxxxxx 14 digits
cardType bpgas(track2 t2)
{
	char tmp2[80];
	cardType card;
	memset(tmp2,0,80);

	//is 16 characters
	if(strlen(t2.fields[0]) !=14) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 7211 constant
	if(strncmp(t2.fields[0],"7211",4)!=0) return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][2])) return card;

	//WE ARE GOOD!
	card.setName("British Petroleum (BP) Gasoline Card");
	strncpy(tmp2,&t2.fields[0][4],10);
	strncpy(&tmp2[10],&t2.fields[1][7],4);
	card.add("Account Number",formatter("XXX XXX XXX X XXXX",tmp2));
	memset(tmp2,0,80);
	strncpy(tmp2,&t2.fields[1][2],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, t2.fields[1][0], t2.fields[1][1]);
	card.add("Expires",tmp2);

	card.setNotes("Cards are Issued and Managed by Citibank");
	card.setUnknowns("Store Number or State it was issued in, Member Since");
	return card;
}

//Delta Crown Room Club Card
//4xxxxxxxxxx=MMYY
cardType deltacrc(track2 t2)
{
	char tmp2[80];
	cardType card;
	memset(tmp2,0,80);

	//1st has is 11 characters
	if(strlen(t2.fields[0]) !=11) return card;
	//2nd has 4 characters
	if(strlen(t2.fields[1]) !=4) return card;
	//Account number passes mod10
	if(!mod10check(&t2.fields[0][1])) return card;
	//starts with a 4
	if(t2.fields[0][0]!='4') return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][0])) return card;

	//WE ARE GOOD!
	card.setName("Delta Crown Room Club Card");
	card.add("Membership Number",formatter("XXX XXX XXXX", &t2.fields[0][1]));
	memset(tmp2,0,80);
	strncpy(tmp2,&t2.fields[1][0],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, t2.fields[1][2], t2.fields[1][3]);
	card.add("Expires",tmp2);

	return card;
}

//Home Depot Consumer Credit Card
//6035xxxxxxxxxxxx@=XXXX101xxxxxxxx 16 digits
cardType homedepotcc(track2 t2)
{

	cardType card;

	//1st is 16
	if(strlen(t2.fields[0])!=16) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//has 101 constant
	if(strncmp(&t2.fields[1][4],"101",3)!=0) return card;
	//starts with a 6035
	if(strncmp(t2.fields[0],"6035",4)) return card;

	//WE ARE GOOD!
	card.setName("Home Depot Consumer Credit Card");
	card.add("Account Number",formatter("XXXX XXXX XXXX XXXX",t2.fields[0]));

	card.setNotes("Card never expires. Is operated and Managed by Citibank");
	card.setUnknowns("Store Number or State it was issued in");
	return card;
}

//Neiman Marcus Charge Card
//3XXXXXXXX= 9 digits
cardType neimanmarcus(track2 t2)
{

	cardType card;

	//1st is 9
	if(strlen(t2.fields[0])!=9) return card;
	//2nd is 0
	if(strlen(t2.fields[1])!=0) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//starts with a 3
	if(t2.fields[0][0]!='3') return card;

	//WE ARE GOOD!
	card.setName("Neiman Marcus Charge Card");
	card.add("Account Number",formatter("XXXX XXXX X",t2.fields[0]));

	card.setUnknowns("Full prefix for card, Store Number or State it was issued in");
	card.setNotes("These Charge Cards have no expiration date, which is rare for charge cards");
	return card;
}

//COSTCO Business Membership Card
//7001XXXXXXXXXXXX=24127XXXXXXXXXXXXXXX 16 digits
cardType costcob(track2 t2)
{

	cardType card;
	char tmp[3] = {0,0,0};

	//1st is 16
	if(strlen(t2.fields[0])!=16) return card;
	//2ns is at least 7
	if(strlen(t2.fields[0])<7) return card;
	//1st starts with a 7001
	if(strncmp(t2.fields[0],"7001",4)!=0) return card;
	//2nd starts with 2412799
	if(strncmp(t2.fields[1],"24127",5)!=0) return card;
	//WE ARE GOOD!
	card.setName("COSTCO Membership Card");
	card.add("Membership Number",formatter("XXXX XXXX XXXX",&t2.fields[0][4]));
	strncpy(tmp,&t2.fields[1][5],2);

	card.add("Member Since",tmp);

	card.setUnknowns("Member Since, Type of card (Business or Standard");
	card.setNotes("These cards are either Standard or Business Class. Stripe Snoop cannot currently tell the difference");

	return card;
}

//US Air Frequent Traveler Card
//7XXXXXXXX=XXXXXXXX 9 digits
cardType usair(track2 t2)
{

	cardType card;
	char tmp2[20];
	memset(tmp2,0,20);


	//1st is 9
	if(strlen(t2.fields[0])!=9) return card;
	//2nd is 8
	if(strlen(t2.fields[1])!=8) return card;
	//starts with a 7
	if(t2.fields[0][0]!='7') return card;
	//has a valid month
	if(!isMonth(&t2.fields[1][2])) return card;

	//WE ARE GOOD!
	card.setName("US Air Frequent Traveler Card");
	card.add("Membership Number",formatter("XXX XXX XXX",t2.fields[0]));
	strncpy(tmp2,&t2.fields[1][2],2); //grab month
	strcpy(tmp2, monthName(atoi(tmp2)));
	//get the year
	sprintf(tmp2,"%s \'%c%c",tmp2, t2.fields[1][0], t2.fields[1][1]);
	card.add("Member Since",tmp2);

	card.setUnknowns("When this card was issued");

	return card;
}

//Walden Books Reader Card
//5XXXXXXXX= 9 digits
cardType walden(track2 t2)
{

	cardType card;

	//1st is 9
	if(strlen(t2.fields[0])!=9) return card;
	//2nd is 0
	if(strlen(t2.fields[1])!=0) return card;
	//passes mod10
	if(!mod10check(t2.fields[0])) return card;
	//starts with a 3
	if(t2.fields[0][0]!='5') return card;

	//WE ARE GOOD!
	card.setName("Walden Books Reader Card");
	card.add("Account Number",formatter("XXX XXX XXX",t2.fields[0]));

	return card;
}


//Checks for AAMVA Driver's License
//636XXXxxxxxxxxx@=YYMMCCYYMMDD? 16 digits

cardType aamva(track2 t2)
{
	char day[3]={0,0,0};
	char year[5]={0,0,0,0,0};

	int i,j;
	FILE * fin;
	char tmp2[80];
	cardType card;
	memset(tmp2,0,80);

	//1st is at least 7 characters
	if(strlen(t2.fields[0]) < 7) return card;
	//2nd is at least 12 characters
	if(strlen(t2.fields[0]) < 12) return card;
	//starts with "636"
	if(strncmp(t2.fields[0],"636",3)!=0) return card;
 	//has a valid month for expire, or 77 88 99
	if(!isMonth(&t2.fields[1][2]) && strncmp(&t2.fields[1][2],"77",2)!=0
		&& strncmp(&t2.fields[1][2],"88",2)!=0
		&& strncmp(&t2.fields[1][2],"99",2)!=0)
			return card;
	//has a valid DOB
	//funny month fix
	if(t2.fields[1][8]!='1')
		t2.fields[1][8] = '0';

	if(!dateValid(&t2.fields[1][8], &t2.fields[1][10], &t2.fields[1][4])) return card;
	//WE ARE GOOD!
	card.setName("AAMVA Compliant North American Driver's License");

	/*get issuing State/Territory!*/
	if( (fin=fopen(AAMVADATABASE,"r")) == NULL) {
		printf("Error: aamva(): Cannot open \"%s\"\n",AAMVADATABASE);
	} else {
		memset(tmp2,0,80);
		bool flag = false;
		while(!feof(fin))
		{
			fgets(tmp2,80,fin);
			if(strlen(tmp2)>8) {
				if( strncmp(t2.fields[0],tmp2,6) == 0) {
					flag = true;
					break;
				}
			}
		}
		//kill trailing CRLF
		tmp2[6 + strlen(&tmp2[7])]=0;
		if(flag)
			card.add("Issuing Territory", &tmp2[7]);
		else
			card.add("Issuing Territory", "Unknown");
		fclose(fin);
	}
	card.add("License Number",&t2.fields[0][6]);

	//get the DOB
	memset(tmp2,0,80);
	strncpy(tmp2,&t2.fields[1][8],2); //grab month
	strncpy(day,&t2.fields[1][10],2);
	strncpy(year,&t2.fields[1][4],4);
	i = atoi(tmp2);
	sprintf(tmp2,"%s %s, %s",monthName(i),day,year);
	card.add("Date of Birth",tmp2);


	//Get the expiration Date
	memset(year,0,5);
	strncpy(year,&t2.fields[1][0],2);;
	j=atoi(year);
	j = expandYear(j);
	if(strncmp(&t2.fields[1][2],"77",2)==0) {
		strcpy(tmp2,"Never");
	} else if(strncmp(&t2.fields[1][2],"88",2)==0) {
		sprintf(tmp2,"%s %d, %d",monthName(i),lastDotm(i,j),j);
	} else if(strncmp(&t2.fields[1][2],"99",2)==0) {
		sprintf(tmp2,"%s %s, %d",monthName(i),day,j);
	} else {
		//plain expiration date
		strncpy(tmp2,&t2.fields[1][2],2); //grab month
		sprintf(tmp2,"%s %d",monthName(atoi(tmp2)),j);
	}
	card.add("Expires",tmp2);

	card.setNotes("This Standard is deployed not only in the US, but parts of Canada as well. Some States/Territories may use letters in their License Numbers. If this is the case, these numbers will be encoded using 2 numbers as follows: A=01, B=02 ... Z=26.");

	return card;
}



// ================================================================= 4 Fields

//Checks for Georgia Institute of Technology Buzzcard
//OLD: 1570=xxxSSNxxx=0x=60177000xxxxxxxx 00 or 02
//NEW: 1570=90xxxxxxx=00=60177000xxxxxxxx
//PARK or TEMP 1570=000000000=00=60177000xxxxxxxx

cardType buzzcard(track2 t2)
{
	cardType card;

	//first field is 1570
	if(strcmp(t2.fields[0],"1570") !=0) return card;
	//2nd field is 9 characters
	if(strlen(t2.fields[1]) !=9) return card;
	//3rd field ==00 || 02
	if( strcmp(t2.fields[2],"00")!=0 && strcmp(t2.fields[2],"02")!=0 )
		return card;
	//4th field starts with 60177000
	if(strncmp(t2.fields[3],"60177000",8) !=0) return card;

	//WE ARE GOOD!
	//which type, old, new, or parking/temp?
	if(strncmp(t2.fields[1],"90",2)!=0) {
		card.setName("Georgia Tech Buzzcard - Pre 2002 version");
		card.add("SSN/Student Number",formatter("XXX-XX-XXXX",t2.fields[1]));
	} else if(strncmp(t2.fields[1],"000000000",9)==0) {
		card.setName("Georgia Tech Parking or Temporary Card");
	} else {
		card.setName("Georgia Tech Buzzcard - GTID Version");
		card.add("GTID",formatter("XXX-XX-XXXX",t2.fields[1]));
	}
	card.add("Database ID",t2.fields[3]);
	return card;
}

cardType lookup2(track2 t2)
{
	cardType result;

	//do only the tests that matter
	if(t2.numFields==1)	{

		result = bnreader(t2);
		if(result.numFields>0) return result;
		result = radisson(t2);
		if(result.numFields>0) return result;
	} else if (t2.numFields==2) {
		result = AAA(t2);
		if(result.numFields>0) return result;
		result = visa(t2);
		if(result.numFields>0) return result;
		result = amex(t2);
		if(result.numFields>0) return result;
		result = kroger(t2);
		if(result.numFields>0) return result;
		result = atm(t2);
		if(result.numFields>0) return result;
		result = bngift(t2);
		if(result.numFields>0) return result;
		result = bpgas(t2);
		if(result.numFields>0) return result;
		result = deltacrc(t2);
		if(result.numFields>0) return result;
		result = homedepotcc(t2);
		if(result.numFields>0) return result;
		result = neimanmarcus(t2);
		if(result.numFields>0) return result;
		result = costcob(t2);
		if(result.numFields>0) return result;
		result = usair(t2);
		if(result.numFields>0) return result;
		result = walden(t2);
		if(result.numFields>0) return result;
		result = aamva(t2);
		if(result.numFields>0) return result;

	} else if (t2.numFields==4) {
		result = buzzcard(t2);
		if(result.numFields>0) return result;

	}

	return result;
}

